function [M,r]=invrec(A,V);

%--------------------------------------------------------------------------------------------
% invrec for a recursive inversion of a symetric matrix.
% This approach allows to inverse, at each iteration, a matrix with a complexity of N^2 instead of N^3.
% In same time we test if the inversion is numericaly possible.
% Arguments:
% A is the previous inverse matrix (N-1)x(N-1)
% V is a line vector (N) which is added to the matrix (the inverse of A) as its last line/column.
% Return: M the new inverse of the matrix (NxN), r is equal to its rank if its was invertible.
% If the inversion failed M=A, and r=N-1
%--------------------------------------------------------------------------------------------
% Copyright  Gaston Baudat & Fatiha Anouar October 2001. Exton PA 19341 USA
% gaston.baudat@eu.effem.com, fatiha.anouar@effem.com
%--------------------------------------------------------------------------------------------

n=length(V);			% size of V
A12=V(1:n-1)';			% the new last column, minus the nth line.
A22=V(n);				% the left lower corner of the new matrix (before inversion)
tmp1=A*A12;				% temporary variables
tmp2=A12'*tmp1-A22;
if (abs(tmp2)>eps) & (abs(A22)>eps)  % Is  the inversion possible ?
	r=n;							 	% Yes, recursive inversion.
    X12=tmp1/tmp2;
    X22=(1.-A12'*X12)/A22;
	X11=A-tmp1*X12';
	M=[X11,X12;X12',X22];
else									% No, keep the previous inverse and rank.
	r=n-1;
	M=A;
end